; ********************************************************************
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
; 
; See the GNU General Public License for more details.
; 
; ********************************************************************
; 
; By monsoonami
; www.youtube.com/monsoonami
; monsoonami.deviantart.com
; 
; ********************************************************************



(define (script-fu-color-cycle old-image old-drawable frames)

  ; only execute the script if an even number of frames was chosen
  (if (= (remainder frames 2) 0)

    (let* (	(height (car (gimp-image-height old-image)))
		(width (car (gimp-image-width old-image)))
		(new-image (car (gimp-image-new width height RGB)))
		(first-frame (car (gimp-layer-new new-image
							width height
							RGBA-IMAGE
							"frame"
							100
							NORMAL-MODE)))
		(new-selection (car (gimp-channel-new new-image
							width height
							"selection"
							100
							'(0 0 0))))
		(old-selection)
		(float)
		(hue-step (/ 200 frames)) ;360 doesn't work as it should
		(half (/ frames 2)))

	(gimp-image-undo-freeze old-image)
	(gimp-image-undo-disable new-image)

	; save the old image's selection to a channel
	; and copy it to a new channel in the new image
	(set! old-selection (car (gimp-selection-save old-image)))
	(gimp-selection-none old-image)
	(gimp-image-add-channel new-image new-selection 0)
	(gimp-edit-copy old-selection)
	(set! float (car (gimp-edit-paste new-selection 1)))
	(gimp-floating-sel-anchor float)

	; the first frame contains the visible contents of the old image
	(gimp-image-add-layer new-image first-frame 0)
	(gimp-edit-copy-visible old-image)
	(set! float (car (gimp-edit-paste first-frame 1)))
	(gimp-floating-sel-anchor float)

	; load the selection in the new image
	; so the colors are only cycled in the selected portion
	(gimp-selection-load new-selection)

	; cycle the hue using the specified number of layers
	(let loop ((i 1))
	  (if (< i frames)
	    (let* (	(hue-offset (round (* (- (remainder (+ i half) frames) half) hue-step)))
			(new-frame (car (gimp-layer-copy first-frame 0))))
		(gimp-image-add-layer new-image new-frame 0)
		(gimp-hue-saturation new-frame 0 hue-offset 0 0)
		(loop (+ i 1)))))

	; restore the old image's selection and cleanup
	(gimp-selection-load old-selection)
	(gimp-selection-none new-image)
	(gimp-image-remove-channel old-image old-selection)
	(gimp-image-remove-channel new-image new-selection)
	(gimp-image-undo-thaw old-image)
	(gimp-image-undo-enable new-image)
	(gimp-display-new new-image)
	(gimp-displays-flush))))


(script-fu-register
  "script-fu-color-cycle"
  "<Image>/Filters/Animation/C_olor Cycle"
  "Create an animation that shifts the selected portion of the image's hue in each frame."
  "monsoonami"
  "monsoonami"
  "August 2011"
  "RGB*"
  SF-IMAGE "Image" 0
  SF-DRAWABLE "Drawable" 0 ;not used
  SF-ADJUSTMENT "Frames (must be even)" '(8 2 60 2 2 0 1))